-- tolua: package class
-- Written by Waldemar Celes
-- TeCGraf/PUC-Rio
-- Jul 1998
-- $Id: package.lua,v 1.5 2000/01/27 11:53:14 celes Exp $

-- This code is free software; you can redistribute it and/or modify it.
-- The software provided hereunder is on an "as is" basis, and
-- the author has no obligation to provide maintenance, support, updates,
-- enhancements, or modifications. 


$debug


-- Package class
-- Represents the whole package being bound.
-- The following fields are stored:
--    {i} = list of objects in the package.
classPackage = {
 _base = classContainer,
 type = 'package'
}
settag(classPackage,tolua_tag)

-- Print method
function classPackage:print ()
 print("Package: "..self.name)
 local i=1
 while self[i] do
  self[i]:print("","")
  i = i+1
 end
end

function classPackage:preprocess ()
 -- perform global substitution
 self.code = " "..self.code    -- to avoid dealing with the begining of string

 self.code = gsub(self.code,"(//[^\n]*)","")     -- eliminate C++ comments
 self.code = gsub(self.code,"/%*","\1")
 self.code = gsub(self.code,"%*/","\2")
 self.code = gsub(self.code,"%b\1\2","")
 self.code = gsub(self.code,"\1","/%*")
 self.code = gsub(self.code,"\2","%*/")
 self.code = gsub(self.code,"%s*@%s*","@") -- eliminate spaces beside @
 self.code = gsub(self.code,"%s?inline(%s)","%1") -- eliminate 'inline' keyword
 self.code = gsub(self.code,"%s?extern(%s)","%1") -- eliminate 'extern' keyword
 self.code = gsub(self.code,"%s?virtual(%s)","%1") -- eliminate 'virtual' keyword
 self.code = gsub(self.code,"public:","") -- eliminate 'public:' keyword
 self.code = gsub(self.code,"([^%w_])void%s*%*","%1_userdata ") -- substitute 'void*'
 self.code = gsub(self.code,"([^%w_])void%s*%*","%1_userdata ") -- substitute 'void*'
 self.code = gsub(self.code,"([^%w_])char%s*%*","%1_cstring ")  -- substitute 'char*'

 self.code = gsub(self.code,"%$%[","\1") -- deal with embedded Lua code
 self.code = gsub(self.code,"%$%]","\2")
end

-- translate verbatim
function classPackage:preamble ()
 output('/*\n')
 output('** Lua binding: '..self.name..'\n')
 output('** Generated automatically by '..TOLUA_VERSION..' on '..date()..'.\n')
 output('*/\n\n')

 if not flags.h then
  output('/* Exported function */')
  output('int  tolua_'..self.name..'_open (void);')
  output('void tolua_'..self.name..'_close (void);')
  output('\n')
 end

 if flags.a then
  output('/* Automatic initialization for C++ code */\n')
  output('#ifdef __cplusplus\n')
  output('static int dummy = tolua_'..self.name..'_open ();')
  output('#endif\n\n')
 end
 output('#include "tolua.h"\n\n')
 local i=1
 while self[i] do
  self[i]:preamble()
  i = i+1
 end
 output('\n')
 output('/* tag variables */')
 self:decltag()
 output('\n')
 output('/* function to register type and initialize tag */')
 output('static void toluaI_init_tag (void)')
 output('{')
 foreach(_usertype,function(n,v) output(' tolua_usertype("',v,'");') end)
 foreach(_tagnames,function(n,v) output(' tolua_settag("',v,'",&'..n,');') end)
 output('}')
 output('\n')
end

-- register package
-- write package open function
function classPackage:register ()
 output("/* Open function */")
 output("int tolua_"..self.name.."_open (void)")
 output("{")
 output(" tolua_open();")
 output(" lua_beginblock();")
 output(" toluaI_init_tag();")
 local i=1
 while self[i] do
  self[i]:register()
  i = i+1
 end
 output(" lua_endblock();")
 output(" return 1;")
 output("}")
end

-- unregister package
-- write package close function
function classPackage:unregister ()
 output("/* Close function */")
 output("void tolua_"..self.name.."_close (void)")
 output("{")
 local i=1
 while self[i] do
  self[i]:unregister()
  i = i+1
 end
 output("}")
end

-- write header file
function classPackage:header ()
 output('/*\n') output('** Lua binding: '..self.name..'\n')
 output('** Generated automatically by '..TOLUA_VERSION..' on '..date()..'.\n')
 output('*/\n\n')

 if not flags.h then
  output('/* Exported function */')
  output('int  tolua_'..self.name..'_open (void);')
  output('void tolua_'..self.name..'_close (void);')
  output('\n')
 end
end

-- Internal constructor
function _Package (t)
 t._base = classPackage
 settag(t,tolua_tag)
 return t
end

-- Constructor
-- Expects the base file name.
-- It assumes the file has extension ".pkg".
function Package (name)
 -- read file
 local code = read('.*')
 -- deal with include directive
 local nsubst
 repeat
  code,nsubst = gsub(code,"%$<(.-)>%s*\n",function (fn)
                                           local fp,msg = openfile(fn,'r')
                                           if not fp then
                                            error('#'..msg..': '..fn)
                                           end
                                           local s = read(fp,'.*')
                                           closefile(fp)
                                           return s
                                          end)
 until nsubst==0

 local t = _Package(_Container{name=name, code=code})
 push(t)
 t:preprocess()
 t:parse(t.code)
 pop()
 return t
end


