/* tolua
** Support code for Lua bindings.
** Written by Waldemar Celes
** TeCGraf/PUC-Rio
** Jul 1998
** $Id: tolua.c,v 1.3 1999/07/28 22:21:09 celes Exp $
*/

/* This code is free software; you can redistribute it and/or modify it. 
** The software provided hereunder is on an "as is" basis, and 
** the author has no obligation to provide maintenance, support, updates,
** enhancements, or modifications. 
*/

#include "tolua.h"
#include "lua.h"
#include "lualib.h"

#include <stdio.h>
#include <stdlib.h>

static void help (void)
{
 fprintf(stderr,"\n"
         "usage: tolua [options] input_file\n"
         "\n"
         "Command line options are:\n"
         "  -v       : print version information.\n"
         "  -o  file : set output file; default is stdout.\n"
         "  -H  file : create include file.\n"
         "  -n  name : set package name; default is input file root name.\n"
         "  -a       : choose auto package initialization (for C++ only).\n"
         "  -p       : parse only.\n"
         "  -P       : parse and print structure information (for debug).\n"
         "  -h       : print this message.\n"
         "Should the input file be omitted, stdin is assumed;\n"
         "in that case, the package name must be explicitly set.\n\n" 
        );
}

static void version (void)
{
 fprintf(stderr, "%s (written by W. Celes)\n",TOLUA_VERSION);
}

static void setfield (char* f, char* v)
{
 lua_pushobject(lua_getglobal("flags"));
 lua_pushstring(f);
 lua_pushstring(v);
 lua_settable();
}

static void error (char* o)
{
 fprintf(stderr,"tolua: unknown option '%s'\n",o);
 help();
 exit(1);
}

int main (int argc, char* argv[])
{
 lua_open();
 lua_iolibopen();
 lua_strlibopen();
 lua_pushstring(TOLUA_VERSION); lua_setglobal("TOLUA_VERSION");

 if (argc==1)
 {
  help();
  return 0;
 }
 else
 {
  int i;
  lua_pushobject(lua_createtable());
  lua_setglobal("flags");
  for (i=1; i<argc; ++i)
  {
   if (*argv[i] == '-')
   {
    switch (argv[i][1])
    {
     case 'v': version(); return 0;
     case 'h': help(); return 0;
     case 'p': setfield("p",""); break;
     case 'P': setfield("P",""); break;
     case 'a': setfield("a",""); break;
     case 'o': setfield("o",argv[++i]); break;
     case 'n': setfield("n",argv[++i]); break;
     case 'H': setfield("H",argv[++i]); break;
     default: error(argv[i]); break;
    }
   }
   else
   {
    setfield("f",argv[i]);
    break;
   }
  }
 }

#if 1

 #include "tolualua.h"

#else

 lua_dofile("basic.lua");
 lua_dofile("feature.lua");
 lua_dofile("verbatim.lua");
 lua_dofile("code.lua");
 lua_dofile("typedef.lua");
 lua_dofile("container.lua");
 lua_dofile("package.lua");
 lua_dofile("module.lua");
 lua_dofile("define.lua");
 lua_dofile("enumerate.lua");
 lua_dofile("declaration.lua");
 lua_dofile("variable.lua");
 lua_dofile("array.lua");
 lua_dofile("function.lua");
 lua_dofile("operator.lua");
 lua_dofile("class.lua");
 lua_dofile("doit.lua");

#endif
 return 0;
}
